import { useEffect, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import { getItem, reset, setItem } from '../../lib/localStorage';
import { appConstants } from '../../lib/AppConstants';
import {
  getSelectedSeatsApi,
  editSeatsApi,
  createSeatApi,
  saveSeatSelectionCountApi,
  getSeatSelectionCountApi
} from '../../lib/api';
import BookingWrapper from '../../components/Layout/BookingWrapper/Index';
import Header from '../../components/Layout/Header/Index';
import ConfirmSeatContent from '../../components/TopLevel/ConfirmSeatContent/Index';
import SeatSelectionContent from '../../components/TopLevel/SeatSelectionContent/Index';
import SuccessModal from '../../components/TopLevel/SuccessModal/Index';
import Button from '../../components/Base/Button/Index';
import LegendWrapper from '../../components/TopLevel/LegendWrapper/Index';
import img from '../../assets/images/power-button_12080802.png';
import styles from './styles.module.css';
const data = [
  ['*', 1, 2, 3, '*', 4, 5, 6, '*'],
  [1, 2, 3, 4, '*', 5, 6, 7, 8],
  [1, 2, 3, 4, '*', 5, 6, 7, 8],
  [1, 2, 3, 4, '*', 5, 6, 7, 8],
  [1, 2, 3, 4, '*', 5, 6, 7, 8],
  [1, 2, 3, 4, '*', 5, 6, 7, 8],
  [1, 2, 3, 4, '*', 5, 6, 7, 8],
  ['*', 1, 2, 3, '*', 4, 5, 6, '*']
];
const BookingPage = () => {
  let userId = getItem(appConstants.USER);
  const navigate = useNavigate();

  const [selectedSeats, setSelectedSeats] = useState(); //persisted seats state
  const [currentSeats, setCurrentSeats] = useState([]); // currently selecting seats before booking(local)
  const [showSeatsModal, setShowSeatsModal] = useState(
    getItem(appConstants.MODAL)
  );
  const [showSeatsConfirmationModal, setShowSeatsConfirmationModal] =
    useState(false);
  const [noOfSeats, setNoOfSeats] = useState();
  const [buttonState, setButtonState] = useState(true);
  const [modalButtonState, setModalButtonState] = useState(true);
  const [showSuccessModal, setShowSuccessModal] = useState(false);
  useEffect(() => {
    const fetchSeatSelectionCount = async () => {
      const data = await getSeatSelectionCountApi(userId);
      setNoOfSeats(data?.selectionCount);
    };
    fetchSelectedSeats();
    fetchSeatSelectionCount();
    setShowSeatsModal(getItem(appConstants.MODAL));
  }, []);

  const fetchSelectedSeats = async () => {
    const data = await getSelectedSeatsApi();
    const currentData = data?.find((e) => e.userid == userId);
    const structuredData = data.flatMap((user) => user.selected);
    const temp = structuredData?.filter(
      (e) =>
        !currentData?.selected?.some(
          (e2) => e2.row === e.row && e2.seat === e.seat
        )
    );
    setSelectedSeats(temp);
    setCurrentSeats(currentData?.selected);
  };

  const logout = () => {
    reset();
    navigate('/login');
  };

  const onModalChange = (value) => {
    const numericValue = parseInt(value, 10);
    setNoOfSeats(numericValue);
    if (value && value !== '') {
      setModalButtonState(false);
    } else {
      setModalButtonState(true);
    }
  };

  const modalClose = () => {
    setItem('modal', false);
    if (userId && noOfSeats) {
      saveSeatSelectionCountApi(userId, noOfSeats);
    }
    fetchSelectedSeats();
    setShowSeatsModal(false);
  };

  const finalCancel = () => {
    setShowSeatsConfirmationModal(false);
  };

  const handleSeatRemove = (rowId, columnId) => {
    setCurrentSeats((prev) => {
      let updatedSeats;
      updatedSeats = prev?.filter(
        (e) => !(e.row === rowId && e.seat === columnId)
      );
      if (updatedSeats?.length === parseInt(noOfSeats)) {
        setButtonState(false);
      } else {
        setButtonState(true);
      }
      return updatedSeats;
    });
  };

  const handleSeatAdd = (rowId, columnId) => {
    setCurrentSeats((prev) => {
      let updatedSeats;
      if (prev?.length < noOfSeats || !prev?.length) {
        if (prev) {
          updatedSeats = [...prev, { row: rowId, seat: columnId }];
        } else {
          updatedSeats = [{ row: rowId, seat: columnId }];
        }
        if (updatedSeats?.length === parseInt(noOfSeats)) {
          setButtonState(false);
        } else {
          setButtonState(true);
        }
      } else {
        updatedSeats = prev ? [...prev] : [];
      }
      return updatedSeats;
    });
  };

  const handleSeatsConfirm = () => {
    setShowSeatsConfirmationModal(true);
  };

  const seatsFinal = async () => {
    let data = await getSelectedSeatsApi();
    const existingUserSeatMap = data?.find((ele) => ele.id == userId);
    if (existingUserSeatMap) {
      await editSeatsApi(userId, currentSeats);
    } else {
      let info = {
        id: userId,
        userid: userId,
        selected: currentSeats
      };
      await createSeatApi(info);
    }
    setShowSeatsConfirmationModal(false);
    fetchSelectedSeats();
    setShowSuccessModal(true);
  };

  const handleSuccessModalConfirm = () => {
    setShowSuccessModal(false);
    setButtonState(true);
  };

  return (
    <div className={styles['seat-booking-wrapper']}>
      <img
        src={img}
        className={styles['logout']}
        width={30}
        onClick={() => logout()}
      />
      <div className={styles['screen-wrapper']}>
        <Header>Choose Seats</Header>
        <BookingWrapper
          seatData={data}
          selectedSeats={selectedSeats}
          currentSeats={currentSeats}
          onSeatRemove={handleSeatRemove}
          onSeatAdd={handleSeatAdd}
        />
      </div>
      <div className={styles['btn-wrapper']}>
        <Button size={'lg'} onClick={handleSeatsConfirm} disabled={buttonState}>
          Confirm
        </Button>
      </div>
      <LegendWrapper />
      {showSeatsModal && (
        <SeatSelectionContent
          onChange={onModalChange}
          onConfirm={modalClose}
          disabled={modalButtonState}
        />
      )}
      {showSeatsConfirmationModal && (
        <ConfirmSeatContent onConfirm={seatsFinal} onCancel={finalCancel} />
      )}
      {showSuccessModal && (
        <SuccessModal onConfirm={handleSuccessModalConfirm} />
      )}
    </div>
  );
};

export default BookingPage;
